import { createRoot } from "react-dom/client";
import App from "./App";
// Import CSS asynchronously to prevent render blocking
// The critical CSS is already inlined in the HTML
import { preloadGoogleMapsApi } from "./lib/google-maps-loader";
import { Layout } from "@/components/layout"; // Added import for Layout component

// Font loading detection
const detectFonts = () => {
  // If the browser supports the font loading API, check that fonts are loaded
  if ('fonts' in document) {
    // This adds redundancy with the inline script, ensuring fonts are tracked correctly
    document.fonts.ready.then(() => {
      document.documentElement.classList.remove('fonts-loading');
      document.documentElement.classList.add('fonts-loaded');
    }).catch(() => {
      // If fonts fail to load, still remove loading class to show content
      document.documentElement.classList.remove('fonts-loading');
    });
  }
};

// Load non-critical CSS asynchronously
const loadNonCriticalCSS = () => {
  // Import the non-critical CSS
  import("./index.css")
    .then(() => {
      console.debug('Non-critical CSS loaded');
      // After CSS is loaded, ensure font detection runs
      detectFonts();
    })
    .catch(e => console.error("Failed to load non-critical CSS:", e));
};

// Load CSS after component hydration
if (document.readyState === 'complete') {
  loadNonCriticalCSS();
} else {
  window.addEventListener('load', loadNonCriticalCSS);
}

// Create a root only once
const rootElement = document.getElementById("root");
if (rootElement) {
  const root = createRoot(rootElement);
  
  // Render the app
  root.render(<App />);
  
  // Mark as loaded to hide the initial loader after React has rendered
  setTimeout(() => {
    rootElement.classList.add('loaded');
  }, 100); // Small delay to ensure React has fully rendered
}

// Register service worker but only in production
// This prevents development mode conflicts
if (import.meta.env.PROD && 'serviceWorker' in navigator) {
  window.addEventListener('load', async () => {
    try {
      // Path to service worker needs to be relative to base URL
      // Use the compiled JavaScript file instead of TypeScript
      const registration = await navigator.serviceWorker.register('/service-worker.js', {
        type: 'module'
      });
      console.log('ServiceWorker registration successful with scope:', registration.scope);
    } catch (error) {
      console.error('ServiceWorker registration failed:', error);
    }
  });
}

// Prefetch critical resources to improve load time
function prefetchResources() {
  // Use API endpoints and URL patterns that will be handled by the Vite dev server
  // or properly built in production
  const resources = [
    '/api/rate-limit-status'
    // Don't prefetch TypeScript files directly
  ];

  // Function to preload third-party APIs with error handling
  const preloadAPIs = async () => {
    try {
      // Check if we're using our global loader
      if (window.loadGoogleMapsApi) {
        // Only preload if not already loaded
        if (!window.google?.maps) {
          try {
            await window.loadGoogleMapsApi();
          } catch (error) {
            // Silently fail on preload - we'll retry when actually needed
            console.debug('Google Maps preloading deferred:', error);
          }
        }
      } else {
        // Fallback to the module-based loader
        preloadGoogleMapsApi();
      }
      
      // reCAPTCHA has been replaced with honeypot field
    } catch (error) {
      // Catch any uncaught errors in the preloading process
      console.debug('Error during API preloading:', error);
    }
  };

  if ('requestIdleCallback' in window) {
    window.requestIdleCallback(() => {
      // First prefetch API endpoints
      resources.forEach(resource => {
        const link = document.createElement('link');
        link.rel = 'prefetch';
        link.href = resource;
        document.head.appendChild(link);
      });

      // Then preload third-party APIs
      preloadAPIs();
    });
  } else {
    // Fallback for browsers that don't support requestIdleCallback
    setTimeout(() => {
      resources.forEach(resource => {
        const link = document.createElement('link');
        link.rel = 'prefetch';
        link.href = resource;
        document.head.appendChild(link);
      });

      // Preload APIs after a delay to prioritize main content
      setTimeout(() => preloadAPIs(), 2000);
    }, 1000);
  }
}

// Call prefetch after the main content has loaded
window.addEventListener('load', prefetchResources);

// Assuming App.tsx uses Router and routes
// This is a reconstruction based on the provided context, actual App.tsx might differ
//  It is important to replace this with your actual App.tsx file content

//This is a guess of what the App.tsx might look like.  Replace this with your actual App.tsx content.
//import { Router, Route } from "wouter";
//import Estimator from "./Estimator";
//import LeadForm from "./LeadForm";
//import NotFound from "./NotFound";
//
//function App() {
//  return (
//    <Router>
//      <Layout>
//        <Route path="/" component={Estimator} />
//        <Route path="/lead-form" component={LeadForm} />
//        <Route component={NotFound} />
//      </Layout>
//    </Router>
//  );
//}
//export default App;